@@ -112,7 +112,7 @@ async def authenticate_message(request, message_tree, message_payload, fingerpri
         connection_fingerprint = utils.get_cert_fingerprint_from_request(request)
         if connection_fingerprint is None:
             msg = ("Your request must use a client side SSL certificate, of which the "
-                   "fingerprint must match the fingerprint that you have given to this VTN")
+                   "fingerprint must match the fingerprint that you have given to this VTN.")
             raise errors.NotRegisteredOrAuthorizedError(msg)
 
         try:
@@ -132,8 +132,8 @@ async def authenticate_message(request, message_tree, message_payload, fingerpri
             raise errors.NotRegisteredOrAuthorizedError(msg)
 
         if connection_fingerprint != expected_fingerprint:
-            msg = (f"The fingerprint of your HTTPS certificate {connection_fingerprint} "
-                   f"does not match the expected fingerprint {expected_fingerprint}")
+            msg = (f"The fingerprint of your HTTPS certificate '{connection_fingerprint}' "
+                   f"does not match the expected fingerprint '{expected_fingerprint}'")
             raise errors.NotRegisteredOrAuthorizedError(msg)
 
         message_cert = utils.extract_pem_cert(message_tree)
